//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip’s products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip’s standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP’S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

/**
 * @file
 * @brief This file contains functions related to the push buttons. This file will
 * set up the push-buttons Init, search for a press detection and determine
 * if the press on a rising, faling edge, if the switch is currently held
 * up or down.
 *
 * @defgroup buttons Buttons
 * @ingroup slave
 * @{
 */

#include "../include/include.h"

typedef struct
{
    char debounce;
    char repeat_delay;
    btn_event_t event;
    unsigned int physical_state:1;
    unsigned int physical_state_pv:1;
    unsigned int logical_state:1;
    unsigned int logical_state_pv:1;
    unsigned int repeat_state:1;
    unsigned int :3; // reserved
}button_t;


button_t sw2;
button_t sw3;

/**
 * This function detect button presses and debounce.
 */
static void do_button(button_t *b, char state)
{
    if(state)
        b->physical_state = 0;
    else
        b->physical_state = 1;
    if(b->physical_state)
    {
        // button has fallen
        if(b->debounce)
            b->debounce --; // decrement the debounce timer
        else
        {
            b->logical_state = 1;
            b->event = BTN_DOWN;
        }
    }
    else // button is !pressed
    {
        if(b->debounce < DEBOUNCE_COUNTER)
            //b->debounce ++;
            b->debounce = DEBOUNCE_COUNTER;
        else
        {
            b->logical_state = 0;
            b->event = BTN_UP;
        }
    }
    // is this a new state?
    if(b->logical_state != b->logical_state_pv)
    {
        if(b->logical_state)
            b->event = BTN_FALL;
        else
            b->event = BTN_RELEASE;
    }
    b->logical_state_pv = b->logical_state;
    b->physical_state_pv = b->physical_state;
}

/**
 * Sets up buttons
 */
void init_button(button_t *b)
{
    b->event = BTN_UP;
    b->debounce = DEBOUNCE_COUNTER;
    b->logical_state = 0;
    b->logical_state_pv = 0;
    b->physical_state = 0;
    b->physical_state_pv = 0;
    b->repeat_delay = 0;
}

/**
 * Configures buttons
 */
void buttons_init(void)
{
    //external pull ups already added
//    SW2_WPU = 1;

    SW2_DIR = 1;
    SW3_DIR = 1;




    init_button(&sw2);
    init_button(&sw3);

}

/**
 * This function checks for a buttons press.
 */
void buttons_update(void)
{
    // manage the buttons

    do_button(&sw2,SW2_IN);
    do_button(&sw3,SW3_IN);
}

/**
 * This function looks for up button presses.
 */
btn_event_t sw2_poll(void)
{
    return sw2.event;
}

btn_event_t sw3_poll(void)
{
    return sw3.event;
}
/**
 * This function looks for down button presses.
 */
//btn_event_t sw7_poll(void)
//{
//    return sw7.event;
//}

/**
 * This function looks for Select button presses.
 */
//btn_event_t sw6_poll(void)
//{
//    return sw6.event;
//}

//btn_event_t sw5_poll(void)
//{
//    return sw5.event;
//}
/** @}*/
